@@ -793,7 +793,7 @@ class Chart(xmlwriter.XMLwriter):
             'diamond': 'diamond',
             'triangle': 'triangle',
             'x': 'x',
-            'star': 'start',
+            'star': 'star',
             'dot': 'dot',
             'short_dash': 'dot',
             'dash': 'dash',
